Package: finetune
Title: Additional Functions for Model Tuning
Version: 1.3.0
Authors@R: c(
    person("Max", "Kuhn", , "max@posit.co", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-2402-136X")),
    person("Posit Software, PBC", role = c("cph", "fnd"),
           comment = c(ROR = "03wc8by49"))
  )
Description: The ability to tune models is important. 'finetune' enhances
    the 'tune' package by providing more specialized methods for finding
    reasonable values of model tuning parameters.  Two racing methods
    described by Kuhn (2014) <doi:10.48550/arXiv.1405.6974> are included.
    An iterative search method using generalized simulated annealing
    (Bohachevsky, Johnson and Stein, 1986)
    <doi:10.1080/00401706.1986.10488128> is also included.
License: MIT + file LICENSE
URL: https://github.com/tidymodels/finetune,
        https://finetune.tidymodels.org
BugReports: https://github.com/tidymodels/finetune/issues
Depends: R (>= 4.1), tune (>= 2.1.0)
Imports: cli, dials (>= 0.3.0), dplyr (>= 1.1.1), ggplot2, parsnip (>=
        1.1.0), purrr (>= 1.0.0), rlang (>= 1.1.0), tibble, tidyr,
        tidyselect, utils, vctrs, workflows (>= 0.2.6)
Suggests: BradleyTerry2, covr, discrim, kknn, klaR, lme4, modeldata,
        ranger, recipes (>= 0.2.0), rpart, rsample, spelling, testthat,
        yardstick
Config/Needs/website: tidyverse/tidytemplate
Config/testthat/edition: 3
Config/usethis/last-upkeep: 2025-05-20
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-04-17 20:09:33 UTC; max
Author: Max Kuhn [aut, cre] (ORCID: <https://orcid.org/0000-0003-2402-136X>),
  Posit Software, PBC [cph, fnd] (ROR: <https://ror.org/03wc8by49>)
Maintainer: Max Kuhn <max@posit.co>
Repository: CRAN
Date/Publication: 2026-04-18 17:00:02 UTC
